@@ -401,8 +401,10 @@ class Study(skrt.core.Archive):
 
 
 class Patient(skrt.core.PathData):
-    """Object associated with a top-level directory whose name corresponds to
-    a patient ID, and whose subdirectories contain studies."""
+    """
+    Object associated with a top-level directory whose name corresponds to
+    a patient ID, and whose subdirectories contain studies.
+    """
 
     def __init__(self, path="", exclude=["logfiles"]):
 
