@@ -23,11 +23,11 @@ class SyntheticImage(Image):
         intensity=-1024,
         noise_std=None,
     ):
-        """Create data to write to a NIfTI file, initially containing a
-        blank image array.
+        """Create an initially blank synthetic image to which geometric 
+        ROIs can be added.
 
-        Parameters
-        ----------
+        **Parameters:**
+        
         shape : int/tuple
             Dimensions of the image array to create in order (x, y, z).
             If an int is given, the image will be created with dimensions
@@ -52,16 +52,15 @@ class SyntheticImage(Image):
             If None, no noise will be applied.
         """
 
-        # Create image properties
+        # Assign properties
         shape = skrt.core.to_list(shape)
         self.shape = [shape[1], shape[0], shape[2]]
-        self.input_voxel_size = [abs(v) for v in skrt.core.to_list(voxel_size)]
-        self.input_origin = origin
+        self.voxel_size = [abs(v) for v in skrt.core.to_list(voxel_size)]
+        self.origin = origin
         self.max_hu = 0 if noise_std is None else noise_std * 3
         self.min_hu = -self.max_hu if self.max_hu != 0 else -20
         self.noise_std = noise_std
         self.bg_intensity = intensity
-        self.background = self.make_background()
         self.shapes = []
         self.roi_shapes = {}
         self.rois = {}
@@ -71,54 +70,79 @@ class SyntheticImage(Image):
         self.rotation = None
 
         # Initialise as Image
-        Image.__init__(self, self.background, 
-                       voxel_size=self.input_voxel_size, 
-                       origin=self.input_origin)
+        Image.__init__(
+            self, self.get_background(), voxel_size=self.voxel_size,
+            origin=self.origin
+        )
 
         # Write to file if a filename is given
         if filename is not None:
             self.filename = os.path.expanduser(filename)
             self.write()
 
-    #  def view(self, **kwargs):
-        #  """View with QuickViewer."""
+    def view(self, **kwargs):
+        """View with QuickViewer."""
 
-        #  from skrt.viewer import QuickViewer
+        from skrt.viewer import QuickViewer
 
-        #  qv_kwargs = {
-            #  "hu": [self.min_hu, self.max_hu],
-            #  "title": "",
-            #  "origin": self.origin,
-            #  "voxel_size": self.voxel_size,
-            #  "mpl_kwargs": {"interpolation": "none"},
-        #  }
-        #  qv_kwargs.update(kwargs)
-        #  rois = self.get_roi_data()
-        #  QuickViewer(self.get_data(), structs=rois, **qv_kwargs)
+        qv_kwargs = {
+            "hu": [self.min_hu, self.max_hu],
+            "title": "",
+            "origin": self.origin,
+            "voxel_size": self.voxel_size,
+            "mpl_kwargs": {"interpolation": "none"},
+        }
+        qv_kwargs.update(kwargs)
+        rois = self.get_roi_data()
+        QuickViewer(self.get_data(), structs=rois, **qv_kwargs)
 
-    def get_data(self):
-        """Get data with noise overlaid."""
+    def update(self, force_bkg=False):
+        """Update self.data so that it contains all current shapes."""
 
-        # Get noiseless image
-        data = self.background.copy()
+        # Get background array
+        data = self.get_background(with_noise=False).copy()
+
+        # Add shapes
         for shape in self.shapes:
             data[shape.get_data(self.get_coords())] = shape.intensity
 
         # Apply noise
-        if self.noise_std is not None:
-            data += np.random.normal(0, self.noise_std, self.shape)
+        data = self.apply_noise(data)
+        self.data = data
 
-        return data
+        # Assign structure set
+        self.clear_structure_sets()
+        self.add_structure_set(self.get_structure_set())
 
-    def plot(self, *args, **kwargs):
-        """Plot the current image."""
+    def get_image(self):
+        """Get self as an Image object."""
 
-        self.sdata = self.get_data()
-        self.structure_sets = [self.get_structure_set()]
-        Image.plot(self, rois=0, *args, **kwargs)
+        self.update()
+        return Image(self)
+
+    def get_data(self, **kwargs):
+        """Get Image data."""
+
+        self.update()
+        return Image.get_data(self, **kwargs)
+
+    def plot(self, **kwargs):
+        """Plot the current image with ROIs overlaid."""
+
+        self.update()
+        self.update_rois()
+        kwargs.setdefault("rois", -1)
+        Image.plot(self, **kwargs)
+
+    def view(self, **kwargs):
+        """View the current image with QuickViewer."""
+
+        from skrt.better_viewer import BetterViewer
+        kwargs.setdefault("rois", self.get_structure_set())
+        return BetterViewer(self.get_image(), **kwargs)
 
     def get_roi_data(self):
-        """Get dict of ROIs and names, with any transformations applied."""
+        """Get dict of ROIs and names with any transformations applied."""
 
         roi_data = {}
         for name, shape in self.roi_shapes.items():
@@ -127,13 +151,10 @@ class SyntheticImage(Image):
         return roi_data
 
     def get_structure_set(self):
-        """Make StructureSet object of own structures."""
+        """Return StructureSet containing own structures."""
 
-        structure_set = StructureSet(name="StructureSet")
         self.update_rois()
-        for roi in self.rois.values():
-            structure_set.add_roi(roi)
-        return structure_set
+        return StructureSet(list(self.rois.values()))
 
     def update_roi(self, name):
         """Update an ROI to ensure it has the correct data."""
@@ -146,9 +167,6 @@ class SyntheticImage(Image):
         for name in self.rois:
             self.update_roi(name)
 
-    def get_structure_sets(self):
-        return [self.get_structure_set()]
-
     def get_roi(self, name):
         """Get a named ROI as an ROI object."""
 
@@ -178,6 +196,7 @@ class SyntheticImage(Image):
                 )
 
         # Write image data
+        self.update()
         Image.write(self, outname)
 
         # Write ROIs
@@ -192,20 +211,36 @@ class SyntheticImage(Image):
         structure_set.write(outdir=outdir, ext=ext_to_use, 
                             overwrite=overwrite_roi_dir)
 
-    def make_background(self):
+    def get_background(self, with_noise=True, force=False):
         """Make blank image array or noisy array."""
 
-        return np.ones(self.shape) * self.bg_intensity
+        bkg = np.ones(self.shape) * self.bg_intensity
+        if with_noise:
+            bkg = self.apply_noise(bkg)
+        return bkg
+
+    def apply_noise(self, array):
+        """Apply background noise to an array."""
+
+        if self.noise_std is not None:
+            array += np.random.normal(0, self.noise_std, array.shape)
+        return array
+
+    def set_noise_std(self, std):
+
+        self.noise_std = std
+        self.update(force_bkg=True)
 
     def reset(self):
         """Remove all shapes."""
 
         self.shapes = []
-        self.roi_shapes = []
+        self.roi_shapes = {}
         self.groups = {}
         self.shape_count = {}
         self.translation = None
         self.rotation = None
+        self.update()
 
     def add_shape(self, shape, shape_type, is_roi, above, group):
 
@@ -225,7 +260,7 @@ class SyntheticImage(Image):
                     self.roi_shapes[group] = self.groups[group]
                     self.rois[group] = ROI(
                         shape.get_data(self.get_coords()), name=group, 
-                        affine=self.affine
+                        affine=self.get_affine()
                     )
                 else:
                     self.groups[group].add_shape(shape)
@@ -242,11 +277,13 @@ class SyntheticImage(Image):
                 self.roi_shapes[shape.name] = shape
                 self.rois[shape.name] = ROI(
                     shape.get_data(self.get_coords()), name=shape.name, 
-                    affine=self.affine
+                    affine=self.get_affine()
                 )
 
         self.min_hu = min([shape.intensity, self.min_hu])
         self.max_hu = max([shape.intensity, self.max_hu])
+        if is_roi:
+            return self.rois[shape.name]
 
     def add_sphere(
         self,
@@ -262,7 +299,7 @@ class SyntheticImage(Image):
         if centre is None:
             centre = self.get_centre()
         sphere = Sphere(self.shape, radius, centre, intensity, name)
-        self.add_shape(sphere, "sphere", is_roi, above, group)
+        return self.add_shape(sphere, "sphere", is_roi, above, group)
 
     def add_cylinder(
         self,
@@ -280,7 +317,7 @@ class SyntheticImage(Image):
         if centre is None:
             centre = self.get_centre()
         cylinder = Cylinder(self.shape, radius, length, axis, centre, intensity, name)
-        self.add_shape(cylinder, "cylinder", is_roi, above, group)
+        return self.add_shape(cylinder, "cylinder", is_roi, above, group)
 
     def add_cube(
         self,
@@ -293,7 +330,7 @@ class SyntheticImage(Image):
         group=None,
     ):
 
-        self.add_cuboid(
+        return self.add_cuboid(
             side_length, centre, intensity, is_roi, name, above, group=group
         )
 
@@ -313,7 +350,7 @@ class SyntheticImage(Image):
         side_length = skrt.core.to_list(side_length)
 
         cuboid = Cuboid(self.shape, side_length, centre, intensity, name)
-        self.add_shape(cuboid, "cuboid", is_roi, above, group)
+        return self.add_shape(cuboid, "cuboid", is_roi, above, group)
 
     def add_grid(
         self, spacing, thickness=1, intensity=0, axis=None, name=None, above=True,
@@ -322,67 +359,6 @@ class SyntheticImage(Image):
         grid = Grid(self.shape, spacing, thickness, intensity, axis, name)
         self.add_shape(grid, "grid", False, above, group=None)
 
-    def get_coords(self):
-        """Get grids of x, y, and z coordinates in mm for this image."""
-
-        if (
-            not hasattr(self, "current_coords")
-            or self.prev_translation != self.translation
-            or self.prev_rotation != self.rotation
-        ):
-
-            # Make coordinates
-            coords_1d = []
-            for i in range(3):
-                coords_1d.append(
-                    np.arange(
-                        self.origin[i],
-                        self.origin[i] + self.voxel_size[i] * self.n_voxels[i],
-                        self.voxel_size[i],
-                    )
-                )
-            X, Y, Z = np.meshgrid(*coords_1d)
-
-            # Apply transformations
-            self.prev_translation = self.translation
-            self.prev_rotation = self.rotation
-            if self.rotation or self.translation:
-                transform = np.identity(4)
-                if self.translation:
-                    transform = transform.dot(
-                        get_translation_matrix(*[-d for d in self.translation])
-                    )
-                if self.rotation:
-                    centre = self.get_centre()
-                    transform = transform.dot(
-                        get_rotation_matrix(*[-r for r in self.rotation], centre)
-                    )
-                Yt = (
-                    transform[0, 0] * Y
-                    + transform[0, 1] * X
-                    + transform[0, 2] * Z
-                    + transform[0, 3]
-                )
-                Xt = (
-                    transform[1, 0] * Y
-                    + transform[1, 1] * X
-                    + transform[1, 2] * Z
-                    + transform[1, 3]
-                )
-                Zt = (
-                    transform[2, 0] * Y
-                    + transform[2, 1] * X
-                    + transform[2, 2] * Z
-                    + transform[2, 3]
-                )
-                X, Y, Z = Xt, Yt, Zt
-
-            # Set coords
-            self.current_coords = (Y, X, Z)
-
-        # Apply transformations
-        return self.current_coords
-
     def reset_transforms(self):
         """Remove any rotations or translations."""
 
@@ -431,8 +407,8 @@ class Sphere:
     def get_data(self, coords):
 
         distance_to_centre = np.sqrt(
-            (coords[0] - self.centre[1]) ** 2
-            + (coords[1] - self.centre[0]) ** 2
+            (coords[1] - self.centre[1]) ** 2
+            + (coords[0] - self.centre[0]) ** 2
             + (coords[2] - self.centre[2]) ** 2
         )
         return distance_to_centre <= self.radius
@@ -450,8 +426,8 @@ class Cuboid:
 
         try:
             data = (
-                (np.absolute(coords[0] - self.centre[1]) <= self.side_length[1] / 2)
-                & (np.absolute(coords[1] - self.centre[0]) <= self.side_length[0] / 2)
+                (np.absolute(coords[1] - self.centre[1]) <= self.side_length[1] / 2)
+                & (np.absolute(coords[0] - self.centre[0]) <= self.side_length[0] / 2)
                 & (np.absolute(coords[2] - self.centre[2]) <= self.side_length[2] / 2)
             )
             return data
@@ -473,16 +449,16 @@ class Cylinder:
     def get_data(self, coords):
 
         # Get coordinates in each direction
-        axis_idx = {"x": 1, "y": 0, "z": 2}[self.axis]
+        axis_idx = _axes.index(self.axis)
         circle_idx = [i for i in range(3) if i != axis_idx]
-        coords_c1 = coords[circle_idx[0]]
-        coords_c2 = coords[circle_idx[1]]
+        coords_c1 = coords[circle_idx[1]]
+        coords_c2 = coords[circle_idx[0]]
         coords_length = coords[axis_idx]
 
         # Get centre in each direction
         centre = [self.centre[1], self.centre[0], self.centre[2]]
-        centre_c1 = centre[circle_idx[0]]
-        centre_c2 = centre[circle_idx[1]]
+        centre_c1 = centre[circle_idx[1]]
+        centre_c2 = centre[circle_idx[0]]
         centre_length = centre[axis_idx]
 
         # Make cylinder array
@@ -506,8 +482,8 @@ class Grid:
     def get_data(self, _):
 
         coords = np.meshgrid(
-            np.arange(0, self.shape[0]),
             np.arange(0, self.shape[1]),
+            np.arange(0, self.shape[0]),
             np.arange(0, self.shape[2]),
         )
         if self.axis is not None:
@@ -518,61 +494,7 @@ class Grid:
             )
         else:
             return (
-                (coords[0] % self.spacing[0] < self.thickness[0])
-                | (coords[1] % self.spacing[1] < self.thickness[1])
+                (coords[1] % self.spacing[1] < self.thickness[1])
+                | (coords[0] % self.spacing[0] < self.thickness[0])
                 | (coords[2] % self.spacing[2] < self.thickness[2])
             )
-
-
-def get_translation_matrix(dx, dy, dz):
-    return np.array([[1, 0, 0, dx], [0, 1, 0, dy], [0, 0, 1, dz], [0, 0, 0, 1]])
-
-
-def get_rotation_matrix(yaw, pitch, roll, centre):
-
-    # Convert angles to radians
-    yaw = np.radians(yaw)
-    pitch = np.radians(pitch)
-    roll = np.radians(roll)
-
-    cx, cy, cz = centre
-    r1 = np.array(
-        [
-            [np.cos(yaw), -np.sin(yaw), 0, cx - cx * np.cos(yaw) + cy * np.sin(yaw)],
-            [np.sin(yaw), np.cos(yaw), 0, cy - cx * np.sin(yaw) - cy * np.cos(yaw)],
-            [0, 0, 1, 0],
-            [0, 0, 0, 1],
-        ]
-    )
-    r2 = np.array(
-        [
-            [
-                np.cos(pitch),
-                0,
-                np.sin(pitch),
-                cx - cx * np.cos(pitch) - cz * np.sin(pitch),
-            ],
-            [0, 1, 0, 0],
-            [
-                -np.sin(pitch),
-                0,
-                np.cos(pitch),
-                cz + cx * np.sin(pitch) - cz * np.cos(pitch),
-            ],
-            [0, 0, 0, 1],
-        ]
-    )
-    r3 = np.array(
-        [
-            [1, 0, 0, 0],
-            [
-                0,
-                np.cos(roll),
-                -np.sin(roll),
-                cy - cy * np.cos(roll) + cz * np.sin(roll),
-            ],
-            [0, np.sin(roll), np.cos(roll), cz - cy * np.sin(roll) - cz * np.cos(roll)],
-            [0, 0, 0, 1],
-        ]
-    )
-    return r1.dot(r2).dot(r3)
